﻿/// <reference path="~/m/js/Vevo.js" />
/// <reference path="~/m/js/Vevo.Objects.js" />
/// <reference path="~/m/js/Vevo.Enums.js" />

Vevo.Helpers = {
	/**
	* Convenience method for creating a namespace.
	*/
	Namespace: function (ns, object) {
		// handle an empty namespace
		if (!ns || ns === "") return window;

		// create the namespace
		var namespaces = ns.split(/\./);
		var current = window;
		_.each(namespaces, function (namespace) {
			console.log("adding " + namespace);
			if (!current[namespace]) {
				current[namespace] = {};
			}
			current = current[namespace];
		});

		if (object) {
			_.extend(current, object);
		}

		return current;
	},

	/**
	* Convenience method for creating a class.
	*/
	Class: function (klass, protoProps, classProps) {
		protoProps = protoProps || {};
		classProps = classProps || {};

		// create the class constructor
		var ctor = function () { };
		var Class = protoProps.constructor || ctor;

		// add the instance methods
		_.extend(Class.prototype, protoProps);

		// add the static methods + a method to subclass this class
		_.extend(Class, classProps);

		// add the extend static method so the class can be subclassed
		// note: this was 'borrowed' from backbone

		// Helper function to correctly set up the prototype chain, for subclasses.
		// Similar to `goog.inherits`, but uses a hash of prototype properties and
		// class properties to be extended.
		var inherits = function (parent, protoProps, staticProps) {
			var child;

			// The constructor function for the new subclass is either defined by you
			// (the "constructor" property in your `extend` definition), or defaulted
			// by us to simply call `super()`.
			if (protoProps && protoProps.hasOwnProperty('constructor')) {
				child = protoProps.constructor;
			} else {
				child = function () { return parent.apply(this, arguments); };
			}

			// Inherit class (static) properties from parent.
			_.extend(child, parent);

			// Set the prototype chain to inherit from `parent`, without calling
			// `parent`'s constructor function.
			ctor.prototype = parent.prototype;
			child.prototype = new ctor();

			// Add prototype properties (instance properties) to the subclass,
			// if supplied.
			if (protoProps) _.extend(child.prototype, protoProps);

			// Add static properties to the constructor function, if supplied.
			if (staticProps) _.extend(child, staticProps);

			// Correctly set child's `prototype.constructor`.
			child.prototype.constructor = child;

			// Set a convenience property in case the parent's prototype is needed later.
			child.__super__ = parent.prototype;

			return child;
		};

		_.extend(Class, { extend: function () {
			var child = inherits(this, protoProps, classProps);
			child.extend = this.extend;
			return child;
		} 
		});

		// create the namespace
		var namespaces = klass.split(/\./);
		klass = namespaces.pop();
		var namespace = this.Namespace(namespaces.join("."));

		// set the class in the correct place in the namespace
		namespace[klass] = Class;

		return Class;
	},

	RelativeTime: function (time_value) {
		//tmp funtion to parse twitter dt
		if (time_value == null) { return ''; }
		var values = time_value.split(" "),
			parsed_date = Date.parse(values[1] + " " + values[2] + ", " + values[5] + " " + values[3]),
			date = new Date(parsed_date),
			relative_to = (arguments.length > 1) ? arguments[1] : new Date(),
			delta = parseInt((relative_to.getTime() - parsed_date) / 1000),
			r = '';

		function formatTime(date) {
			var hour = date.getHours(),
				min = date.getMinutes() + "",
				ampm = 'AM';

			if (hour == 0) {
				hour = 12;
			} else if (hour == 12) {
				ampm = 'PM';
			} else if (hour > 12) {
				hour -= 12;
				ampm = 'PM';
			}

			if (min.length == 1) {
				min = '0' + min;
			}

			return hour + ':' + min + ' ' + ampm;
		}

		function formatDate(date) {
			var ds = date.toDateString().split(/ /),
				mon = monthDict[date.getMonth()],
				day = date.getDate() + '',
				dayi = parseInt(day),
				year = date.getFullYear(),
				thisyear = (new Date()).getFullYear(),
				th = 'th';

			// anti-'th' - but don't do the 11th, 12th or 13th
			if ((dayi % 10) == 1 && day.substr(0, 1) != '1') {
				th = 'st';
			} else if ((dayi % 10) == 2 && day.substr(0, 1) != '1') {
				th = 'nd';
			} else if ((dayi % 10) == 3 && day.substr(0, 1) != '1') {
				th = 'rd';
			}

			if (day.substr(0, 1) == '0') {
				day = day.substr(1);
			}

			return mon + ' ' + day + th + (thisyear != year ? ', ' + year : '');
		}

		delta = delta + (relative_to.getTimezoneOffset() * 60);

		if (delta < 5) {
			r = 'less than 5 seconds ago';
		} else if (delta < 30) {
			r = 'half a minute ago';
		} else if (delta < 60) {
			r = 'less than a minute ago';
		} else if (delta < 120) {
			r = '1 minute ago';
		} else if (delta < (45 * 60)) {
			r = (parseInt(delta / 60)).toString() + ' minutes ago';
		} else if (delta < (2 * 90 * 60)) { // 2* because sometimes read 1 hours ago
			r = 'about 1 hour ago';
		} else if (delta < (24 * 60 * 60)) {
			r = 'about ' + (parseInt(delta / 3600)).toString() + ' hours ago';
		} else {
			if (delta < (48 * 60 * 60)) {
				r = formatTime(date) + ' yesterday';
			} else {
				// r = formatTime(date) + ' ' + formatDate(date);
				r = (parseInt(delta / 86400)).toString() + ' days ago';
			}
		}

		return r;
	},

	/**
	* Calculate the time ago in words.
	*
	* @todo Handle timezones
	* @todo Use an offset from the server's time?
	* @todo Merge with RelativeTime
	*/
	TimeAgoInWords: function (timestamp) {
		var now = (new Date()).getTime();
		var secondsAgo = now - timestamp;

		// < 1 minute
		if (secondsAgo < 60) {
			return "a minute ago";
			// minutes
		} else if (secondsAgo < 3600) {
			return Math.floor(secondsAgo / 60) + " minutes ago";
			// hours
		} else if (secondsAgo < 86400) {
			return Math.floor(secondsAgo / 3600) + " hours ago";
			// days
		} else if (secondsAgo < 604800) {
			return Math.floor(secondsAgo / 86400) + " days ago";
			// weeks
		} else if (secondsAgo < 2592000) {
			return Math.floor(secondsAgo / 604800) + " weeks ago";
			// months
		} else if (secondsAgo < 31536000) {
			return Math.floor(secondsAgo / 2592000) + " months ago";
			// years
		} else {
			return Math.floor(secondsAgo / 31536000) + " years ago";
		}
	},

	CleanHtml: function (string) {
		var tmp = document.createElement("div");
		tmp.innerHTML = string;
		return tmp.textContent || tmp.innerText;
	},

	Cookies: {
		set: function (name, value, days, domain) {
			var expires = "";
			if (days) {
				var date = new Date();
				date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
				expires = "; expires=" + date.toUTCString();
			}
			var cookie = name + "=" + value + expires + "; path=/";
			if (domain) {
				cookie += "; domain=" + domain;
			}
			document.cookie = cookie;

		},
		get: function (name) {
			var nameEQ = name + "=";
			var ca = document.cookie.split(';');
			for (var i = 0; i < ca.length; i++) {
				var c = ca[i];
				while (c.charAt(0) == ' ') c = c.substring(1, c.length);
				if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length, c.length);
			}
			return null;
		},
		getBoolean: function (name) {
			return 'true' == this.get(name) || 'True' == this.get(name);
		},
		remove: function (name) {
			document.cookie = name + "='';expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/";
		}
	},

	Analytics: {
		trackEvent: function (category, action, label) {
			if (typeof _gaq === 'undefined') {
				Vevo.Helpers.Console.Log('trackEvent: _gaq == undefined');
			} else {
				_gaq.push(['_trackEvent', category, action, label, 0, true])
			}
		},
		trackPageview: function (url) {
			if (typeof _gaq === 'undefined') {
				Vevo.Helpers.Console.Log('trackPageview: _gaq == undefined');
			} else {
				_gaq.push(['_trackPageview', url]);
			}
		},
		trackSuperBeacon: function (sourceId, videoId, userId, domain, referrer, region) {
			$.ajax(
				"http://sb.vevo.com/Player/PlayStart.ashx",
				{ data: { sourceId: sourceId, videoId: videoId, userId: userId, domain: domain, referrer: referrer, region: region} }
			);
		},
		trackCustomVar: function (index, name, value, opt_scope) {
			if (typeof _gaq === 'undefined') {
				Vevo.Helpers.Console.Log('trackCustomVar: _gaq == undefined');
			} else {
				_gaq.push(['_setCustomVar', index, name, value, opt_scope]);
			}
		},
		BeginTracking: function () {
			var playerContainer = $("#playerHTML5");
			var video = Vevo.Views.Watch.CurrentVideo;
			Vevo.HTML5.Conviva.beginSessionMonitoring();
			Vevo.Helpers.Analytics.trackEvent("VideoPlayerHtml5", "PlaybackBegin", Vevo.HTML5.Freewheel.theVideoAssetId);
			Vevo.Helpers.Analytics.trackSuperBeacon("e1f36bdf-761c-435a-8fa0-432e39a6f019", Vevo.HTML5.Freewheel.theVideoAssetId, "0", "www.vevo.com", "", "US");
			Vevo.Tracking.ComScore.trackComScore("1", "3005647", "", video.MainArtist != null && video.MainArtist.length > 0 ? video.MainArtist[0].Name : "", "02", "");
		}
	},
	GetHTML5Video: function (authdData, rendition) {
		var source = null;
		rendition = typeof rendition !== 'undefined' ? rendition : "High";
		Vevo.Helpers.Console.Log("GetHTML5Video: Rendition: " + rendition);

		$(authdData.video.videoVersions).each(function (index, version) {
			if (version.sourceType == 2) {
				source = Vevo.Helpers.GetHTML5VideoSourceFromData(version.data, rendition);
			}
		});
		return source;
	},
	GetHTML5VideoSourceFromData: function (data, rendition) {
		var xmlDoc = Vevo.Helpers.XML.CreateXMLDoc(data);
		var renditionNode = Vevo.Helpers.XML.GetElementWithAttributeValue(xmlDoc, "rendition", "name", rendition);
		//var source = renditionNode.getAttribute("url"); 
		var isrc = Vevo.Views.Watch.CurrentVideo.ISRC;
		Vevo.Helpers.Console.Log("ISRC: " + isrc);
		// Mobile Encode Not Working, Attempting to use L3 encode
		var source = "http://prog.lvl3.strm.vevo.com/Video/V2/H264/" + isrc.toString().toUpperCase() + "/" + isrc.toString().toLowerCase() + "_high_480x360_x264_608_quicktime_192.mp4";
		Vevo.Helpers.Console.Log("URL: " + source);
		return source;
	},
	XML: {
		CreateXMLDoc: function (data) {
			var xmlDoc;
			if (window.DOMParser) {
				parser = new DOMParser();
				xmlDoc = parser.parseFromString(data, "text/xml");
			}
			else // Internet Explorer
			{
				xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
				xmlDoc.async = false;
				xmlDoc.loadXML(data);
			}
			return xmlDoc;
		},
		GetElementWithAttributeValue: function (xmlDoc, tagName, attribute, attributeValue) {
			var nodeToReturn = null;
			$(xmlDoc.getElementsByTagName(tagName)).each(function (index, nodeObj) {
				Vevo.Helpers.Console.Log(nodeObj.nodeName + " : " + nodeObj.nodeValue);
				if (nodeObj.getAttribute(attribute) == attributeValue) {
					nodeToReturn = nodeObj;
				}
			});
			return nodeToReturn;
		}
	},
	utf8Encode: function (string) {
		if (typeof string != 'string') {
			return string;
		}

		string = string.replace(/\r\n/g, "\n");
		var utftext = "";

		for (var n = 0; n < string.length; n++) {

			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if ((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}

		}
		return utftext;
	},

	Console:
	{
		Log: function (val) {
			try {
				if (console != null && console != undefined) {
					console.log(val);
				}

			} catch (E) {
				//alert(val);
			}

		}
	},

	replaceURLWithHTMLLinks: function (text, openInNewWindow) {
		var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
		return text.replace(exp, "<a href='$1'" + (openInNewWindow ? "target='_blank'" : "") + ">$1</a>");
	},

	insertCommasInNumber: function (num) {
		num += '';
		var x = num.split('.');
		var x1 = x[0];
		var x2 = x.length > 1 ? '.' + x[1] : '';
		var rgx = /(\d+)(\d{3})/;
		while (rgx.test(x1)) {
			x1 = x1.replace(rgx, '$1' + ',' + '$2');
		}
		return x1 + x2;
	},

	Playlist:
	{
		Create: function () {
			var appendTo = 'body';
			var template = '<div id="playlist-new" class="playlist-rail playlist-rail-tall"></div>';
			if (Vevo.Environment.Context.PageType == Vevo.Enums.PageType.Playlist && $('.btn-new-playlist').length > 0) {
				appendTo = '#playlists';
				$(appendTo).append(template);
			}

			var playlist = new Vevo.Controls.Playlist('#playlist-new');
			playlist.Playlist.Name = Vevo.Resources.Playlists_New;
			playlist.Playlist.Items = [];
			playlist.OnCancelDialog = "$('#playlist-new').remove();";
			playlist.IsNew = true;
			playlist.OnSavePlaylist();
		},
		Exist: function (name) {
			//
			var items = Vevo.Environment.Playlists.Items;
			for (var ix = 0; ix < items.length; ix++) {
				if (items[ix].Name == name) return true;

			}
			return false;
		}
	},

	calculateRank: function (num) {
		num += '';
		var onesDigit = num.substring(num.length - 1, num.length),
			tensDigit = num > 10 ? num.substring(num.length - 2, num.length - 1) : 0;
		if (tensDigit != 1 && onesDigit == 1) {
			num += 'st';
		} else if (tensDigit != 1 && onesDigit == 2) {
			num += 'nd';
		} else if (tensDigit != 1 && onesDigit == 3) {
			num += 'rd';
		} else {
			num += 'th';
		}
		return num;
	},

	Badgeville: {
		InitUser: function () {
			Vevo.Environment.OnLoadedCall(function () {
				if (Vevo.Environment.Context.Email && Vevo.Environment.Context.Email != '' &&
					Vevo.Environment.Context.DisplayName && Vevo.Environment.Context.DisplayName != '') {

					try {
						Badgeville.extend(Badgeville.Settings, {
							key: Vevo.Environment.Context.Badgeville.Key,
							domain: Vevo.Environment.Context.Badgeville.Domain
						});

						Badgeville.setPlayer({
							email: Vevo.Environment.Context.Email,
							display_name: Vevo.Environment.Context.DisplayName
						});

						// Any actions that rely on Badgeville being initialized (such as gear creation) should be wrapped in a Badgeville.ready call
						Badgeville.ready(function () {
							$('#uvTab').click(function () {
								try {
									Badgeville.credit('feedback');
								} catch (e) { }
							});
						});

					} catch (e) { }
				}
			}, this);
		},
		Init: function (badgevilleUrl, badgevilleKey, badgevilleDomain) {
			Vevo.Environment.OnLoadedCall(function () {
				Vevo.Environment.OnFacebookLoadedCall(function () {
					if (Vevo.Environment.Context.Email && Vevo.Environment.Context.Email != '' &&
						Vevo.Environment.Context.DisplayName && Vevo.Environment.Context.DisplayName != '') {

						Vevo.Environment.Context.Badgeville.Key = badgevilleKey;
						Vevo.Environment.Context.Badgeville.Domain = badgevilleDomain;

						window.BadgevilleAsynchInit = Vevo.Helpers.Badgeville.InitUser;

						(function () {
							var s = document.createElement('script');
							s.async = true;
							s.src = document.location.protocol + badgevilleUrl;
							document.body.appendChild(s);
						} ());
					}
				}, this);
			}, this);
		},
		Credit: function (creditAction, metaData) {
			Vevo.Environment.OnLoadedCall(function () {
				try {
					if (Badgeville !== undefined && Badgeville && Badgeville.ready !== undefined && Badgeville.ready &&
						Vevo.Environment.Context.Loaded && Vevo.Environment.Context.IsLoggedIn) {

						Badgeville.ready(function () {
							if (!!metaData) {
								var m = metaData;
								m.verb = creditAction;
								Badgeville.credit(m);
							} else {
								Badgeville.credit(creditAction);
							}
						});
					}
				} catch (e) { }
			}, this);
		}
	},

	Window: {
		QueryStringValue: function (name) {
			var arr = (new RegExp('[\\?&]' + name + '=([^&#]*)')).exec(window.location.search);
			return arr != null && arr.length > 0 ? arr[arr.length - 1] : null;
		},
		HashStringValue: function (name) {
			var arr = (new RegExp('[\\#?&]' + name + '=([^&#]*)')).exec(window.location.hash);
			return arr != null && arr.length > 0 ? arr[arr.length - 1] : null;
		}
	}
};

/////extend vevo object
//var tmp = $.extend(false, Vevo.Helpers, VevoHelpers);
//Vevo.Helpers = tmp;
//VevoObjects = null;



