﻿/// <reference path="~/m/js/Vevo.js" />
/// <reference path="~/m/js/Vevo.Objects.js" />
/// <reference path="~/m/js/Vevo.API.js" />
/// <reference path="~/m/js/Vevo.Helpers.js" />

Vevo.Temp = {
	Helper: {
		CacheKeyPrefix: 'v_1_',
		VideosAsISRC: function (items) {
			var toReturn = [];
			for (var ix = 0, l = items.length; ix < l; ix++) {
				toReturn.push(items[ix].ISRC);
			}
			return toReturn;
		},
		ISRCsAsVideos: function (items) {
			var toReturn = [];
			for (var ix = 0, l = items.length; ix < l; ix++) {
				//toReturn.push(items[ix].ISRC);
				Vevo.Repository.Videos.Get(items[ix], function (item) { toReturn.push(item); });
			}
			return toReturn;
		},
		GetISRC: function (videoItem) {
			var isrc = typeof videoItem == "string" ? videoItem : videoItem.ISRC;
			return isrc;
		}
	},
	Videos: {
		Get: function (isrc, onComplete) {
			var cacheKey = this.CacheKey(isrc);
			var item = Vevo.Cache.Get(cacheKey);

			//item = null;
			if (item == null) {
				//video not in cache
				item = Vevo.API.LoadVideo(isrc, function (item) { Vevo.Cache.Add(cacheKey, item); if (onComplete) { onComplete(item); } });
			} else {
				//notify the calling function that we are done
				onComplete(item);
			}
		},
		Authenticate: function (authUrl, isrc, domain, authToken, pKey, callback) {
			var cacheKey = "auth:" + isrc;
			var item = Vevo.Cache.Get(cacheKey);
			Vevo.Helpers.Console.Log("AuthCacheItem:");
			Vevo.Helpers.Console.Log(item);
			if (item == null || true) {
				Vevo.Helpers.Console.Log("Calling Vevo.API.AuthenticateVideo");
				Vevo.API.AuthenticateVideo(authUrl, isrc, domain, authToken, pKey, callback);
			} else {
				Vevo.Helpers.Console.Log("Calling callback");
				Vevo.Helpers.Console.Log(item);
				callback(item);
			}
		},
		CacheAuthentication: function (isrc, item) {
			var cacheKey = "auth:" + isrc;
			Vevo.Cache.Add(cacheKey, item);
		},
		CacheKey: function (isrc) { return 'video:' + isrc }
	},
	News: {
		Get: function (artistSafeName, onComplete, baseObj) {
			var cacheKey = Vevo.Repository.Helper.CacheKeyPrefix + 'news_' + artistSafeName;
			var item = Vevo.Cache.Get(cacheKey);
			if (item == null) {
				item = Vevo.API.LoadNews(artistSafeName, function (item) {
					item = new Vevo.Objects.VideoItem.Clone(item)
					if (item.News.Loaded && (item.News.Items.length > 1 || (item.News.Items.length == 1 && item.News.Items[0].Title == ''))) {
						//only cache news
						Vevo.Cache.Add(cacheKey, item.News);
					}


					if (onComplete) { onComplete(item); }
				}, baseObj);
			} else {
				//set news back to base obj
				baseObj.News = item;
				//notify the calling function that we are done
				onComplete(baseObj);
			}
		}
	},
	Lyrics: {
		Get: function (isrc, onComplete, baseObj) {
			var cacheKey = Vevo.Repository.Helper.CacheKeyPrefix + 'lyrics_' + isrc;
			var item = Vevo.Cache.Get(cacheKey);
			//item = null;
			if (item == null) {
				item = Vevo.API.LoadLyrics(isrc, function (item) {
					item = new Vevo.Objects.VideoItem.Clone(item);

					//only cache lyrics
					Vevo.Cache.Add(cacheKey, item.Lyrics);

					if (onComplete) { onComplete(item); }
				}, baseObj);
			} else {
				baseObj.Lyrics = item;
				//notify the calling function that we are done
				onComplete(baseObj);
			}
		}
	},
	Artist: {
		Get: function (artistSafeName, onComplete, baseObj) {
			var cacheKey = Vevo.Repository.Helper.CacheKeyPrefix + 'artist_' + artistSafeName;
			var item = Vevo.Cache.Get(cacheKey);
			//item = null;
			if (item == null) {
				item = Vevo.API.LoadArtistInfo(artistSafeName, function (item) {
					item = new Vevo.Objects.VideoItem.Clone(item);
					//item.MainArtist[0].Videos = Vevo.Repository.Helper.VideosAsISRC(item.MainArtist[0].Videos);
					Vevo.Cache.Add(cacheKey, item.MainArtist);


					if (onComplete) { onComplete(item); }
				}, baseObj);
			} else {
				//item.Videos = Vevo.Repository.Helper.ISRCsAsVideos(item.MainArtist[0].Videos);
				baseObj.MainArtist = item;
				//notify the calling function that we are done
				onComplete(baseObj);
			}
		}
	},
	Playlist:
	{
		CacheKey: function (id) { return Vevo.Repository.Helper.CacheKeyPrefix + 'playlist_' + id },
		Echonest: function (isrc, refPlaylist, onComplete) {
			var cacheKey = Vevo.Repository.Playlist.CacheKey(isrc);
			var item = Vevo.Repository.Playlist.GetFromCache(cacheKey);

			if (item == null) {
				//item = Vevo.API.LoadArtistInfo(artistSafeName, function (item) { Vevo.Cache.Add(cacheKey, item); onComplete(item); }, baseObj);

				item = Vevo.API.EchonestPlaylist('video', isrc, refPlaylist, function (item) {
					Vevo.Repository.Playlist.SaveToCache(item, cacheKey);
					if (onComplete) {
						onComplete(item);
					}
				});
			} else {
				item = new Vevo.Objects.Playlist.Clone(item);
				//notify the calling function that we are done
				onComplete(item);
			}

			//Vevo.API.NewPlaylist('video', isrc, refPlaylist, onComplete);
		},
		ArtistMix: function (artistUrlSafeName, refPlaylist, onComplete) {
			var cacheKey = Vevo.Repository.Playlist.CacheKey(artistUrlSafeName) + ':mixitup:true';
			var item = Vevo.Repository.Playlist.GetFromCache(cacheKey);
			if (item == null) {
				//item = Vevo.API.LoadArtistInfo(artistSafeName, function (item) { Vevo.Cache.Add(cacheKey, item); onComplete(item); }, baseObj);

				item = Vevo.API.GetArtistMixPlaylist(artistUrlSafeName, function (item) {
					Vevo.Repository.Playlist.SaveToCache(item, cacheKey);
					if (onComplete) {
						onComplete(item);
					}
				}, orderByRecentVideos);
			} else {
				item = new Vevo.Objects.Playlist.Clone(item);
				//notify the calling function that we are done
				onComplete(item);
			}

			//Vevo.API.NewPlaylist('video', isrc, refPlaylist, onComplete);
		},
		Artist: function (artistUrlSafeName, onComplete, orderByRecentVideos) {
			var cacheKey = Vevo.Repository.Playlist.CacheKey(artistUrlSafeName) + ':orderbyrecent:' + orderByRecentVideos;
			var item = Vevo.Repository.Playlist.GetFromCache(cacheKey);
			if (item == null) {
				item = Vevo.API.GetArtistPlaylist(artistUrlSafeName, function (item) {
					Vevo.Repository.Playlist.SaveToCache(item, cacheKey);
					if (onComplete) {
						onComplete(item);
					}
				}, orderByRecentVideos);
			} else {
				item = new Vevo.Objects.Playlist.Clone(item);
				//notify the calling function that we are done
				onComplete(item);
			}
		},
		Get: function (playlistID, onComplete) {
			var cacheKey = Vevo.Repository.Playlist.CacheKey(playlistID);
			var item = Vevo.Repository.Playlist.GetFromCache(cacheKey);
			//item = null;
			if (item == null) {
				item = Vevo.API.GetPlaylist(playlistID, function (item) {
					Vevo.Repository.Playlist.SaveToCache(item);
					if (onComplete) {
						onComplete(item);
					}
				});
			} else {
				item = new Vevo.Objects.Playlist.Clone(item);
				//notify the calling function that we are done
				onComplete(item);
			}

		},
		Delete: function (playlistID, onComplete) {
			var cacheKey = Vevo.Repository.Playlist.CacheKey(playlistID);
			Vevo.API.DeletePlaylist(playlistID, function (item) {
				Vevo.Cache.Delete(cacheKey);
				//refresh in memory cache
				Vevo.Repository.Playlist.AddPlaylistToUserCache(null);
				//Holding off until we find a better way to do this
				//Vevo.Facebook.OGMusic.Playlist.unpublish(playlistUrl);
				if (onComplete) {
					onComplete(item);
				}
			});

		},
		ByUser: function (userID, onComplete) {
			/// <summary>Retrieves all playlists for specified user</summary>
			/// <param name="userID" type="int">UserID of the Vevo User or leave blank to retrieve playlist of the current identity</param>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>

			if (Vevo.Environment.Context.IsLoggedIn == false) {
				//user not logged in
				return;
			}

			var cacheKey = "playlists";
			if (userID != undefined && userID != null) {
				cacheKey += ":" + userID;
			}
			var item = Vevo.Cache.Get(cacheKey);
			//item = null;
			if (item == null) {
				Vevo.API.GetPlaylistByUserID(userID,
					function (playlists) {
						var pIDs = [];
						//add the data to cache
						for (var ix = 0; ix < playlists.length; ix++) {
							pIDs.push(playlists[ix].SessionID);
							//Vevo.Cache.Add('playlist:' + playlists[ix].SessionID, playlists[ix]);
							Vevo.Repository.Playlist.SaveToCache(playlists[ix]);
						}
						//cache a list of ids
						if (pIDs.length > 0) {
							Vevo.Cache.Add(cacheKey, pIDs);
						}
						if (userID == null) {
							$(Vevo.Environment.Playlists).trigger("change", { items: playlists });
						}


						if (onComplete) { onComplete(playlists); }
					}
				);
			} else {
				var playlists = item;
				var toReturn = [];
				var pIDs = [];
				for (var ix = 0; ix < playlists.length; ix++) {
					var tmp = Vevo.Repository.Playlist.GetFromCache(Vevo.Repository.Playlist.CacheKey(playlists[ix]), false);
					if (tmp != null) {
						pIDs.push(tmp.SessionID);
						toReturn.push(tmp);
					}
				};
				if (userID == null) {
					$(Vevo.Environment.Playlists).trigger("change", { items: toReturn });
				}
				Vevo.Cache.Add(cacheKey, pIDs);

				//notify the calling function that we are done
				if (onComplete) onComplete(toReturn);
			}

		},
		AddPlaylistToUserCache: function (playlist) {
			/// <summary>Adds playlist to the list of user playlists in cache</summary>
			/// <param name="playlist" type="Vevo.Objects.Playlist">Playlist Object</param>
			var cacheKey = "playlists"
			var item = Vevo.Cache.Get(cacheKey);
			if (item != null) {
				var pIDs = [];
				var playlists = item;
				var toReturn = [];
				var skipAdd = false; //means we already updated it
				for (var ix = 0; ix < playlists.length; ix++) {

					var tmp = Vevo.Repository.Playlist.GetFromCache(Vevo.Repository.Playlist.CacheKey(playlists[ix]), false)
					if (tmp != null) {
						if (playlist != null && playlist.SessionID != '' && playlist.SessionID == tmp.SessionID) {
							//the list that was added already exists in the collection, just update collection
							skipAdd = true;
							pIDs.push(tmp.SessionID);
							toReturn.push(playlist);
						} else {
							pIDs.push(tmp.SessionID);
							toReturn.push(tmp);
						}
					}
				};


				if (playlist != null && skipAdd == false) {
					pIDs.push(playlist.SessionID);
					toReturn.push(playlist);
				}
			}
			Vevo.Cache.Add(cacheKey, pIDs);
			$(Vevo.Environment.Playlists).trigger("change", { items: toReturn, trigger: playlist });
		},

		//save function
		Save: function (playlist, onComplete) {
			/// <summary>Inserts new playlist's into DB, Updates DB if its an existing playlist.</summary>
			/// <param name="playlist" type="Vevo.Objects.Playlist">Playlist Object</param>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>
			Vevo.API.SavePlaylist(playlist, function (item) {

				Vevo.Repository.Playlist.SaveToCache(item, null);

				Vevo.Repository.Playlist.AddPlaylistToUserCache(item);

				//Publishing or updating a playlist on facebook
				if (playlist.SessionID) {
					Vevo.Facebook.OGMusic.Playlist.update(item.ShortUrl);
					Vevo.Helpers.Badgeville.Credit('edit-playlist');
				} else {
					Vevo.Facebook.OGMusic.Playlist.publish(item.ShortUrl);
					Vevo.Helpers.Badgeville.Credit('create-playlist');
				}
				if (onComplete) {
					onComplete(item);
				}
			});
		},
		//save function
		SaveToCache: function (playlist, cacheKey) {
			/// <summary>Save's playlist into Browser Session Storage</summary>
			/// <param name="playlist" type="Vevo.Objects.Playlist">Playlist Object</param>
			/// <param name="cacheKey" type="string">Cache Key that playlist is stored under. Passing in null, will result in default playlist name.</param>
			if (cacheKey == null || cacheKey == undefined) {
				cacheKey = Vevo.Repository.Helper.CacheKeyPrefix + 'playlist_' + playlist.SessionID;
			}
			playlist = new Vevo.Objects.Playlist.Clone(playlist);
			playlist.Items = playlist.ItemsAsISRC();
			Vevo.Cache.Add(cacheKey, playlist);

			//Vevo.Repository.Playlist.AddPlaylistToUserCache(playlist);
		},
		GetFromCache: function (cacheKey, preloadVideos) {
			/// <summary>Retrieves playlist from Browser Session Storage</summary>
			/// <param name="cacheKey" type="string">Cache Key that playlist is stored under</param>
			if (preloadVideos == null || preloadVideos == undefined) preloadVideos = true;
			try {
				var item = Vevo.Cache.Get(cacheKey);
				if (item != null && item.Items != null) {
					if (item.Items.length > 0 && preloadVideos) {
						item.Items = Vevo.Repository.Helper.ISRCsAsVideos(item.Items);

					}
					return item;
				}
			} catch (e) {
				Vevo.Helpers.Console.Log(e);
				Vevo.Helpers.Analytics.trackEvent('Error', 'Cache', e);
			}


			return null;
		},
		CreatePlaylistBasedOnArtistsNames: function (artists, playlistName, isFacebook, onComplete) {
			var query = '';

			if (null != artists && artists.length > 0) {
				for (var i = 0, l = artists.length; i < l; i++) {
					query += (query == '' ? '' : ',') + (artists[i].name ? artists[i].name : artists[i]);
				}
			}

			//call api for matching artists
			$.ajax({
				url: '/data/videosbyartistmatch/',
				data: jQuery.param({ 'artistNames': query, 'isFacebook': isFacebook }),
				type: 'POST',
				success: function (data, status, xhr) {
					var p = new Vevo.Objects.Playlist.Clone(Vevo.Objects.Playlist);
					p.Name = playlistName;
					p.Items = [];
					if (data && data.length > 0) {
						for (var i = 0, l = data.length; i < l; i++) {
							var v = new Vevo.Objects.VideoItem.Clone(Vevo.Objects.VideoItem);

							v = Vevo.API.ParseVideo(data[i], v);

							p.Items.push(v);
						}
					}

					if (onComplete) {
						onComplete(p);
					}
				}
			});
		},

		//this is for the format returned by the api (versus the data controller)
		CreatePlaylistBasedOnAPIVideoObjects: function (videos, playlistName, onComplete) {
			var p = new Vevo.Objects.Playlist.Clone(Vevo.Objects.Playlist);
			p.Name = playlistName;
			p.Items = [];
			if (videos && videos.length > 0) {
				for (var i = 0, l = videos.length; i < l; i++) {
					var v = new Vevo.Objects.VideoItem.Clone(Vevo.Objects.VideoItem);
					v.ISRC = videos[i].isrc;
					v.Title = videos[i].title;
					v.ViewCount = videos[i].viewcount;
					v.IsPremiere = videos[i].premiere;
					v.Img = videos[i].image_url;

					p.Items.push(v);
				}
			}

			if (onComplete) {
				onComplete(p);
			}
		},
		DeleteEmpty: function () {
			for (var ix = 0; ix < Vevo.Environment.Playlists.Items.length; ix++) {
				var p = Vevo.Environment.Playlists.Items[ix];
				if (p.Items.length == 0) {
					Vevo.Helpers.Console.Log("Deleting Empty Playlist:" + p.Name);
					Vevo.Helpers.Console.Log(p);
					Vevo.Repository.Playlist.Delete(p.SessionID, function (obj) { Vevo.Helpers.Console.Log("Deleted:" + obj); });
				}
			}
		}
	},

	ScaniTunes: function (successCallBack, callbackFor300, callbackFor301, callbackFor_1) {
		var iTunesFinished = false,
			iTunesFailed = false,
			iTunes = new iTunesScan({
				"applet_id": "iTunesLibrary",
				"successCallback": function () {
					if (!iTunesFailed) {
						iTunesFinished = true;
						successCallBack();
					}
				},
				"eventCallback": function (code, msg) {
					switch (parseInt(code)) {
						case 300:
							//started scanning iTunes - messaging doesn't appear to work, holding here for potential integration in the future
							setTimeout(function () {
								//we add this timeout for cases where the applet doesn't throw a -1 message, but doesn't complete either
								if (!iTunesFinished || !iTunesFailed) {
									iTunesFailed = true;
									if (callbackFor_1) {
										callbackFor_1(code, msg);
									}
								}
							}, 15000);
							if (callbackFor300) {
								callbackFor300(code, msg);
							}
							break;
						case 301:
							//progress of scanning iTunes - doesn't give meaningful information other than progressing
							if (callbackFor301) {
								callbackFor301(code, msg);
							}
							break;
						case 400:
							//total number of playlists in iTunes
							break;
						case 401:
							//current progress of parsing iTunes playlists
							break;
						case -1:
							//means that no iTunes library has been found
							iTunesFailed = true;
							if (callbackFor_1) {
								callbackFor_1(code, msg);
							}
							break;
						default:
							break;
					}
				}
			});

		iTunes.start();
		return iTunes;
	},

	Facebook: {
		GetFriends: function (facebookData, onComplete) {
			var cacheKey = 'friends',
				item = Vevo.Cache.Get(cacheKey);

			if (item == null) {
				$.ajax({
					type: 'GET',
					url: 'https://graph.facebook.com/' + facebookData.providerId + '/friends?access_token=' + facebookData.secretKey + '&callback=?',
					dataType: 'json',
					success: function (json) {
						if (json && json.data && json.data.length > 0) {
							var ids = new Array();

							for (var i = 0, l = json.data.length; i < l; i++) {
								ids[i] = json.data[i].id + '|' + json.data[i].name;
							}

							$.ajax({
								type: 'POST',
								url: '/account/findfriends',
								data: 'friendIds=' + ids.join(','),
								dataType: 'json',
								success: function (json) {
									if (json && json.length > 0) {
										Vevo.Cache.Add(cacheKey, json);
										onComplete(json);
									} else if (json && json.error && json.error.type == "OAuthException") {
										onComplete(null);
									}
								}
							});
						} else if (json && json.error && json.error.type == "OAuthException") {
							Vevo.Repository.Facebook.UpdateAccessToken(facebookData, function (d) {
								Vevo.Repository.Facebook.GetFriends(d, onComplete);
							});
							//                        } else if (json && json.data && json.data.length == 0) {
							//                            //not getting an auth error, but invalid access token - weird side case
							//                            facebookData.providerId = FB.getAuthResponse().userID;
							//                            facebookData.secretKey = FB.getAuthResponse().accessToken;
							//                            Vevo.Repository.Facebook.GetFriends(facebookData, onComplete);
						}
					}
				});
			} else {
				onComplete(item);
			}
		},
		GetAllFriends: function (facebookData, onComplete) {
			var cacheKey = 'all_friends',
				item = Vevo.Cache.Get(cacheKey);

			if (item == null) {

				Vevo.API.GetAllFriends(FB.getAuthResponse().accessToken, facebookData.userId, onComplete);

				//				$.ajax({
				//					type: 'GET',
				//					url: 'https://graph.facebook.com/' + facebookData.providerId + '/friends?access_token=' + facebookData.secretKey + '&callback=?',
				//					dataType: 'json',
				//					success: function (json) {
				//						if (json && json.data && json.data.length > 0) {
				//							var ids = new Array();

				//							for (var i = 0, l = json.data.length; i < l; i++) {
				//								ids[i] = json.data[i].id + '|' + json.data[i].name;
				//							}
				//							onComplete(json);

				//						} else if (json && json.error && json.error.type == "OAuthException") {
				//							Vevo.Repository.Facebook.UpdateAccessToken(facebookData, function (d) {
				//								Vevo.Repository.Facebook.GetAllFriends(d, onComplete);
				//							});
				//						} else if (json && json.data && json.data.length == 0) {
				//							//not getting an auth error, but invalid access token - weird side case
				//							facebookData.providerId = FB.getAuthResponse().userID;
				//							facebookData.secretKey = FB.getAuthResponse().accessToken;
				//							Vevo.Repository.Facebook.GetAllFriends(facebookData, onComplete);
				//						}
				//					}
				//				});
			} else {
				onComplete(item);
			}
		},

		GetMusicLikes: function (facebookData, onComplete) {
			$.ajax({
				type: 'GET',
				url: 'https://graph.facebook.com/' + facebookData.providerId + '/music?access_token=' + facebookData.secretKey + '&callback=?',
				dataType: 'json',
				success: function (json) {
					if (json && json.data) {
						if (onComplete) {
							onComplete(json);
						}
					} else if (json && json.error && json.error.type == "OAuthException") {
						Vevo.Repository.Facebook.UpdateAccessToken(facebookData, function (d) {
							Vevo.Repository.Facebook.GetMusicLikes(d, onComplete);
						});
					}
					//removed by ilya, was causing user with no likes on fb to get stuck in the loop
					//
					//					} else if (json && json.data && json.data.length == 0) {
					//						//not getting an auth error, but invalid access token - weird side case
					//						facebookData.providerId = FB.getAuthResponse().userID;
					//						facebookData.secretKey = FB.getAuthResponse().accessToken;
					//						Vevo.Repository.Facebook.GetMusicLikes(facebookData, onComplete);
					//					}
				}
			});
		},

		UpdateAccessToken: function (facebookData, onComplete) {
			//access token has expired, get new token
			$.ajax({
				url: '/account/updateaccesstoken/',
				type: 'POST',
				success: function (data, status, xhr) {
					Vevo.Environment.Context.IsLoggedIn = data.loggedIn;
					Vevo.Environment.Context.UserID = data.userId;
					Vevo.Environment.Context.DisplayName = data.name;
					Vevo.Environment.Context.FacebookId = data.providerId;
					Vevo.Environment.Context.Email = data.email;
					if (onComplete) {
						onComplete(data);
					}
				}
			});
		}
	},
	Search: function (req, resp) {
		/// <summary>This function is designed to work with jQuery.autocomplete</summary>
		/// <param name="req" type="string">Requst Object</param>
		/// <param name="resp" type="function"">Response Function</param>
		var timeStart,
			triedLucine = false,
			maxArtists = 20,
			maxItemsToShow = 40;

		function onSuccess(data) {
			if (data.Artists == undefined) {
				data = JSON.parse(data);
			}
			var displayedArtists = [], displayedVideos = [], topArtist, topArtistLatest, topArtistAll;
			//track success in Google Analytics
			var gaLocation = Vevo.Environment.Context.PageName == Vevo.Enums.PageType.Home ? 'home' : 'header';

			Vevo.Helpers.Analytics.trackEvent(gaLocation, 'main', 'search');

			var maxArtists = triedLucine ? 3 : 40;

			if (data.Artists && data.Artists.length > 0) {
				displayedArtists = $.map(data.Artists.slice(0, maxArtists), function (anArtist, i) {
					return {
						label: anArtist.name,
						value: anArtist,
						type: "artist",
						category: Vevo.Enums.Search.LabelCategoryArtist
					};
				});
			}
			if (data.Videos) {
				displayedVideos = $.map(data.Videos.slice(0, maxItemsToShow), function (aVideo, i) {
					return {
						label: aVideo.name,
						value: aVideo,
						type: "video",
						category: Vevo.Enums.Search.LabelCategoryVideo
					};
				});
			}
			//call response function with param of list items to be show, check to make sure they're not null
			var newData = [];
			if (topArtist) {
				newData = newData.concat(topArtist);
			}
			if (topArtistLatest) {
				newData = newData.concat(topArtistLatest);
			}
			if (topArtistAll) {
				newData = newData.concat(topArtistAll);
			}
			if (displayedArtists) {
				newData = newData.concat(displayedArtists);
			}
			if (displayedVideos) {
				newData = newData.concat(displayedVideos);
			}
			if (newData.length == 0) {
				Vevo.Helpers.Console.Log('no results');
				Vevo.Helpers.Analytics.trackEvent(gaLocation, 'search-no-results', req.term);
				newData.push({
					label: "No Results",
					value: "",
					type: "artist",
					category: 'NoResults'
				})
				if (!triedLucine) {
					triedLucine = true;
					//doCall('search');
					Vevo.API.Search(req.term, 'search', onSuccess);
				} else {
					resp(newData);
				}

			} else {
				resp(newData);
			}

		}


		Vevo.API.Search(req.term, 'lookahead', onSuccess);
	},
	Account: {
		GetLoggedInStatus: function (onComplete) {
			/// <summary>Gets the current user's logged in status.</summary>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>
			var cacheKey = 'loggedIn',
				item = Vevo.Cache.Get(cacheKey);
			if (!item) {
				Vevo.API.GetLoggedInStatus(function (item) {
					Vevo.Cache.Add(cacheKey, item);

					if (onComplete) {
						onComplete(item);
					}
				});
			} else {
				onComplete(item);
			}
		},
		Login: function (onComplete) {
			/// <summary>Logs the current user in.</summary>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>
			var cacheKey = 'loggedIn';
			Vevo.API.Login(function (item) {
				Vevo.Cache.Delete(cacheKey);

				if (onComplete) {
					onComplete(item);
				}
			});
		},
		Logout: function (onComplete) {
			/// <summary>Logs the current user in.</summary>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>
			var cacheKey = 'loggedIn';
			Vevo.API.Logout(function (item) {
				Vevo.Cache.Delete(cacheKey);

				if (onComplete) {
					onComplete(item);
				}
			});
		},
		InvalidateLoginCache: function (onComplete) {
			/// <summary>Invalidates current cache of logged in status.</summary>
			/// <param name="onComplete" type="function">To be executed at the end of operation</param>
			Vevo.Cache.Delete('loggedIn');

			if (onComplete) {
				onComplete();
			}
		},

		LoginOrCreateUser: function (callback) {
			//if they've authorized VEVO, see if they're logged in - if not, log them in
			Vevo.Repository.Account.GetLoggedInStatus(
				function (data) {
					Vevo.Environment.Context.IsLoggedIn = data.loggedIn;
					if (data.loggedIn && callback) {
						//they're logged in, so continue on
						callback(data);
					} else {
						//find if there is a user associated with that Facebook account
						$.ajax({
							url: '/account/finduserid/',
							type: 'POST',
							data: 'userid=' + FB.getUserID(),
							success: function (data, status, xhr) {
								if (data.userExists) {
									//account with that Facebook id exists, so log them in
									Vevo.Repository.Account.Login(
										function (data) {
											//update their logged in status
											Vevo.Repository.Account.GetLoggedInStatus(
												function (data) {
													Vevo.Environment.Context.IsLoggedIn = data.loggedIn;
													if (data.loggedIn && callback) {
														//continue down register path
														callback(data);
													}
												}
											);
										}
									);
								} else {
									//no user with that Facebook id exists, so we need to create them
									$.ajax({
										type: 'GET',
										url: 'https://graph.facebook.com/' + FB.getUserID() + '?accesstoken=' + FB.getAccessToken() + '&callback=?',
										dataType: 'json',
										success: function (json) {
											if (json) {
												if (!json.email || json.email == undefined) {
													//Facebook graph doesn't always return email, need to use fql then as a backup
													FB.api(
														{
															method: 'fql.query',
															query: 'SELECT email,birthday FROM user WHERE uid=' + FB.getUserID()
														},
														function (r) {
															$.ajax({
																url: '/account/createuser/',
																type: 'POST',
																data: 'userID=' + FB.getUserID() + '&accesstoken=' + FB.getAccessToken() + '&name=' + json.name + '&gender=' + json.gender + '&email=' + r[0].email + '&birthday=' + r[0].birthday,
																success: function (data, status, xhr) {
																	Vevo.Repository.Account.InvalidateLoginCache();
																	Vevo.Repository.Account.GetLoggedInStatus(
																		function (data) {
																			Vevo.Environment.Context.IsLoggedIn = data.loggedIn;
																			if (data.loggedIn && callback) {
																				callback(data);
																			}
																		}
																	);
																}
															});
														}
													);
												} else {
													$.ajax({
														url: '/account/createuser/',
														type: 'POST',
														data: 'userID=' + FB.getUserID() + '&accesstoken=' + FB.getAccessToken() + '&name=' + json.name + '&gender=' + json.gender + '&email=' + json.email + '&birthday=' + json.birthday,
														success: function (data, status, xhr) {
															Vevo.Repository.Account.InvalidateLoginCache();
															Vevo.Repository.Account.GetLoggedInStatus(
															function (data) {
																Vevo.Environment.Context.IsLoggedIn = data.loggedIn;
																if (data.loggedIn && callback) {
																	callback(data);
																}
															}
														);
														}
													});
												}
											}
										}
									});
								}
							}
						});
					}
				}
			);
		}
	}
};

///extend vevo object
var tmp = $.extend(false, Vevo.Repository, Vevo.Temp);
Vevo.Repository = tmp;
Vevo.Temp = null;



